/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.geoviewer.SingleObjectLayer;
import nl.cloudfarming.client.model.EntitySavable;
import org.netbeans.api.actions.Savable;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject.Registration;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

@NbBundle.Messages("field display name=Field")
@Registration(displayName = "#field display name", iconBase = "nl/cloudfarming/client/field/type/field.png", mimeType = Field.MIMETYPE)
public class FieldDataObject extends MultiDataObject implements PropertyChangeListener {

    private InstanceContent ic;
    private Lookup intern;
    private FieldManagerImpl fieldManager;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field");
    private final Field field;

    public FieldDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(Field.MIMETYPE, true);

        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);

        fieldManager = Lookup.getDefault().lookup(FieldManagerImpl.class);
        assert fieldManager != null : "No FieldManager class found in the default lookup, unable to load the DataObject";

        field = fieldManager.loadField(getPrimaryFile());
        assert field != null;
        LOGGER.log(Level.FINE, "FieldDataObject loaded {0}", System.identityHashCode(field));
        ic.add(field);

        LOGGER.log(Level.FINE, "Adding new SingleObjectLayer in FieldDataObject lookup");
        ic.add(new SingleObjectLayer(field.getName(), field, getLookup()));
        ic.add(new FieldSplitHandler(field));
        field.addPropertyChangeListener(this);

    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    protected Node createNodeDelegate() {
        LOGGER.log(Level.FINE, "Creating new node delegate for FieldDataObject");
        return new FieldNode(super.createNodeDelegate(), field);
    }

    @Override
    public void propertyChange(PropertyChangeEvent arg0) {
        Savable savable =
                this.getLookup().
                lookup(Savable.class);
        if (null == savable) {
            this.ic.add(new FieldSavable());
        }
    }

    @Override
    public final Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }

    @Override
    protected FileObject handleRename(String name) throws IOException {
        return super.handleRename(name);
    }

    /**
     * Handles deletion of this FieldDataObject Removes any {@link Savable} and
     * de-registers them
     *
     * @throws IOException
     */
    @Override
    protected void handleDelete() throws IOException {
        super.handleDelete();
        FieldSavable savable = this.getLookup().lookup(FieldSavable.class);
        if (null != savable) {
            savable.destroy();
        }
    }
    private static final Icon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/field/type/field.png", true);

    private class FieldSavable extends EntitySavable {

        public FieldSavable() {
            super(field, ICON);
            register();
        }

        @Override
        public void handleSave() throws IOException {
            dataObject().fieldManager.save(field);
            destroy();
        }

        @Override
        protected String findDisplayName() {
            return field.getName();
        }

        @Override
        public void destroy() {
            dataObject().ic.remove(FieldSavable.this);
            unregister();
        }
        
        @Override
        public void dispose() {
            dataObject().dispose();
        }

        FieldDataObject dataObject() {
            return FieldDataObject.this;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof FieldSavable) {
                FieldSavable m = (FieldSavable) obj;
                return dataObject() == m.dataObject();
            }
            return false;
        }

        @Override
        public int hashCode() {
            return dataObject().hashCode();
        }

    }
}
