/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.field.model.Shape;
import nl.cloudfarming.client.field.model.ShapeCollection;
import nl.cloudfarming.client.field.type.wizard.NewFieldAction;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.util.ProgressListener;
import org.jdesktop.swingworker.SwingWorker;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Timon Veenstra
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-farm")
@NbBundle.Messages({
    "Fields node name=Fields",
    "LBL_import_fields_progress_handle=Importing fields"
})
public class FieldsNodeFactory implements NodeFactory {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field");

    @Override
    public NodeList<?> createNodes(Project p) {
        List<Node> nodes = new ArrayList<>();
        // add fields node to project
        FileObject fieldsFolder = FieldManagerImpl.getFieldsFolder(p, true);
        DataFolder fieldsDataObject = DataFolder.findFolder(fieldsFolder);
        nodes.add(new FieldsNode(fieldsDataObject.getNodeDelegate(), p));

        return NodeFactorySupport.fixedNodeList(nodes.toArray(new Node[nodes.size()]));
    }

    /**
     * This method takes a node as a parameter and checks if the passed node is
     * not null and has a lookup containing a {@link ShapeCollection}. Only
     * nodes that have a ShapeCollection in their lookup can be accepted and
     * processed by the FiedsNode.
     *
     * @param node The node dropped onto the FieldsNode
     * @return true if the passed node is not null and has a ShapeCollection in
     * its lookup
     */
    protected boolean acceptNode(Node node) {
        if (node != null) {
            if (node.getLookup().lookup(ShapeCollection.class) == null) {
                return false;
            }
            return true;
        }
        return false;
    }

    /**
     * Helper method to ease the process of accepting multiple nodes. This
     * method calls acceptNode for each single node of a passed node array and
     * validates the node.
     *
     * @param nodes Array of nodes that should be tested for acceptance
     * @return false if any of the nodes in the passed array does not contain a
     * lookup with a {@link ShapeCollection}, otherwise true
     */
    protected boolean acceptNode(Node[] nodes) {
        for (Node node : nodes) {
            if (!acceptNode(node)) {
                return false;
            }
        }
        return true;
    }

    private final class FieldsNode extends FilterNode {
        private final Project project;

        public FieldsNode(Node original, Project project) {
            super(original, new FieldChildren(original), new ProxyLookup(original.getLookup(), Lookups.fixed(new SimpleLayer(Bundle.Fields_node_name()))));
            this.project = project;
            disableDelegation(DELEGATE_SET_NAME);
            setName(Bundle.Fields_node_name());
            setDisplayName(Bundle.Fields_node_name());
        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage("nl/cloudfarming/client/field/type/field.png");
        }

        @Override
        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        @Override
        public boolean canCopy() {
            return false;
        }

        @Override
        public boolean canRename() {
            return true;
        }

        @Override
        public boolean canCut() {
            return false;
        }

        @Override
        public PasteType getDropType(final Transferable transferable, int action, int index) {
            final FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);
            final Node[] nodes = NodeTransfer.nodes(transferable, NodeTransfer.DND_COPY);

            if (nodes != null && acceptNode(nodes) && fieldManager != null) {
                return new PasteType() {
                    @Override
                    public Transferable paste() throws IOException {
                        processDroppedNodes(nodes, fieldManager);
                        return null;
                    }
                };

            }
            return null;
        }

        /**
         * Uses the {@link FieldManager} and the Node array passed as parameter
         * to import the shapes contained by the {@link ShapeCollection} of the
         * nodes lookup as fields.
         *
         * @param nodes The nodes containing the ShapeCollections
         * @param fieldManager The FieldManager that should be used to perform
         * the import operation
         */
        private void processDroppedNodes(Node[] nodes, final FieldManager fieldManager) {
            List<Shape> shapes = new ArrayList<>();
            for (Node node : nodes) {
                shapes.addAll(node.getLookup().lookup(ShapeCollection.class).getShapes());
            }
            importShapes(shapes, fieldManager);
        }

        /**
         * This method creates a SwingWorker which calls the actual importFields
         * method of the passed FieldManager. Importing fields is a blocking
         * operation and needs to be performed on a separate thread. This method
         * uses a {@link ProgressListener} to display and update a
         * ProgressHandle of the import process.
         *
         * @param shapes The shapes that need to be imported as fields
         * @param fieldManager The FieldManager responsible for the import
         * process
         */
        private void importShapes(final List<Shape> shapes, final FieldManager fieldManager) {
            ProgressHandle progressHandle = ProgressHandleFactory.createHandle(Bundle.LBL_import_fields_progress_handle());
            progressHandle.start(shapes.size());
            final ProgressListener progressListener = new ProgressListener(progressHandle, FieldManager.PROP_PROGRESS, FieldManager.PROP_FINISH);
            SwingWorker swinger = new SwingWorker() {
                @Override
                protected Object doInBackground() throws Exception {
                    fieldManager.importFields(shapes, project, progressListener);
                    return null;
                }
            };
            swinger.execute();
        }

        @Override
        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

        @Override
        public Transferable clipboardCopy() throws IOException {
            return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
        }

        @Override
        public Action[] getActions(boolean context) {
            return new Action[]{new NewFieldAction(project)};
        }
    }

    private static class FieldChildren extends FilterNode.Children {

        public FieldChildren(Node or) {
            super(or);
        }

        @Override
        protected Node[] createNodes(Node key) {
            List<Node> result = new ArrayList<>();

            for (Node node : super.createNodes(key)) {
                if (accept(node)) {
                    result.add(node);
                }
            }

            return result.toArray(new Node[0]);
        }

        // only accept non fileobject nodes or nodes that do not have a 
        // folder fileobject in their lookup
        private boolean accept(Node node) {
            return node.getLookup().lookup(FileObject.class) == null || !node.getLookup().lookup(FileObject.class).isFolder();
        }
    }
}
